<?php

	/**
	 * @author David Grudl [aka dgx]
	 * @author upravil Petr Havelka [aka PHX]
	 * @link http://phx.gipix.net
	 * @version 20.3.2009
	 *
	 * @copyright Copyright (c) 2009 Petr Havelka [aka PHX]
	 */

// =====================================================================================================================

	class DataGrid extends Control {

		/** @persistent */
		public $page = 1;

		/** @persistent */
		public $order = '';

		protected $columns = array();

		protected $canOrder = array();

		protected $cellNotify = array();

		/** @var DibiDataSource */
		protected $dataSource;

		/** @var Paginator */
		protected $paginator;

		/** @var bool */
		private $useAjax = true;

// =====================================================================================================================

		public function __construct(IComponentContainer $parent = null, $name = null) {
			parent::__construct($parent, $name);
			$this->paginator = new Paginator();
			$this->paginator->itemsPerPage = 15;
		}

// =====================================================================================================================

		public function setDataSource(DibiDataSource $dataSource) {
			$this->dataSource = $dataSource;
			$this->paginator->itemCount = count($dataSource);
		}

// =====================================================================================================================

		/**
		 * @return DibiDataSource
		 */
		public function getDataSource() {
			return $this->dataSource;
		}

// =====================================================================================================================

		public function setHiddenColumns($column) {
			if (is_array($column)) {
				foreach ($column as $one) {
					$this->dataSource->select($one);
				}
			}
			else {
				$this->dataSource->select($column);
			}

			return $this;
		}

// =====================================================================================================================

		public function setColumns($column, $title = false) {
			if (is_array($column)) {
				$this->columns = $column;
			}
			else {
				$this->columns[$column] = $title;
			}

			return $this;
		}

// =====================================================================================================================

		public function setDataColumns($column, $title = false, $canOrder = true) {
			if (is_array($column)) {
				$this->columns = $column;
				foreach ($column as $item=>$title) {
					$this->dataSource->select($item);
					$this->canOrder[$item] = true;
				}
			}
			else {
				$this->columns[$column] = $title;
				$this->dataSource->select($column);

				$this->canOrder[$column] = $canOrder;
			}

			return $this;
		}

// =====================================================================================================================

		public function setCellNotify($column, $callback = null) {
			if (is_array($column)) {
				$this->cellNotify = $column;
			}
			else {
				$this->cellNotify[$column] = $callback;
			}

			return $this;
		}

// =====================================================================================================================

		public function handlePage($page) {
//			Debug::fireLog($page);
			$this->invalidateControl('grid');
		}

// =====================================================================================================================

		public function renderPaginator() {
			if ($this->paginator->pageCount < 2) return;

			$this->paginator->page = $this->page;

			// paginator steps
			$arr = range(max($this->paginator->firstPage, $this->page - 3), min($this->paginator->lastPage, $this->page + 3));
			$count = 4;
			$quotient = ($this->paginator->pageCount - 1) / $count;
			for ($i = 0; $i <= $count; $i++) {
				$arr[] = round($quotient * $i) + $this->paginator->firstPage;
			}
			sort($arr);

			// render
			$template = $this->createTemplate();
			$template->paginator = $this->paginator;
			$template->setFile(Environment::expand('%componentTemplateDir%/DataGrid/paginator.phtml'));
			$template->registerFilter('Nette\Templates\CurlyBracketsFilter::invoke');
			$template->useAjax = $this->useAjax;
			$template->steps = array_values(array_unique($arr));
			$template->render();
		}

// =====================================================================================================================

		public function handleOrder($by) {
			parse_str($this->order, $list);

			if (!isset($list[$by])) {
				// ZDE SMAZAT PRO RAZENI DLE VICE SLOUPECKU
				$list = array();
				// END
				$list[$by] = 'a';
			} elseif ($list[$by] === 'd') {
				// ZDE SMAZAT PRO RAZENI DLE VICE SLOUPECKU
				$list[$by] = 'a';
				// END
				// ZDE ODKOMENTOVAT PRO RAZENI DLE VICE SLOUPECKU
//				unset($list[$by]);
				// END
			} else {
				$list[$by] = 'd';
			}

			$this->order = http_build_query($list, '', '&');
		}

// =====================================================================================================================

		public function renderGrid() {
			$dataSource = $this->dataSource;

			// paging
			$this->paginator->page = $this->page;
			$dataSource->applyLimit($this->paginator->length, $this->paginator->offset);

	    // sorting
			$i = 1;
			parse_str($this->order, $list);
			foreach ($list as $field => $dir) {
				$dataSource->orderBy($field, $dir === 'a' ? dibi::ASC : dibi::DESC);
				$list[$field] = array($dir, $i++);
			}

			// render
			$template = $this->createTemplate();
			$template->id = $this->getName();
			$template->rows = $dataSource->getIterator();
			$template->columns = $this->columns;
			$template->canOrder = $this->canOrder;
			$template->cellNotify = $this->cellNotify;
			$template->order = $list;
			$template->useAjax = $this->useAjax;
			$template->setFile(Environment::expand('%componentTemplateDir%/DataGrid/grid.phtml'));
			$template->registerFilter('Nette\Templates\CurlyBracketsFilter::invoke');
			$template->render();
		}

// =====================================================================================================================
// =====================================================================================================================

		public function getItemsPerPage() {
			return $this->paginator->itemsPerPage;
		}

// =====================================================================================================================

		public function setItemsPerPage($value) {
			$this->paginator->itemsPerPage = $value;
		}

// =====================================================================================================================

		public function getUseAjax() {
			return $this->useAjax;
		}

// =====================================================================================================================

		public function setUseAjax($value) {
			$this->useAjax = $value;
		}

// =====================================================================================================================

	}
